@@ -780,7 +780,7 @@ Status SlotMigrator::migrateComplexKey(const rocksdb::Slice &key, const Metadata
       if (metadata.Type() > RedisTypeNames.size()) {
         return {Status::NotOK, "unknown key type: " + std::to_string(metadata.Type())};
       }
-      return {Status::NotOK, "unsupported complex key type: " + metadata.TypeName()};
+      return {Status::NotOK, fmt::format("unsupported complex key type: {}", metadata.TypeName())};
     }
   }
 
@@ -1293,6 +1293,9 @@ Status SlotMigrator::sendSnapshotByRawKV() {
     batch_sender.SetPrefixLogData(log_data);
 
     GET_OR_RET(batch_sender.Put(storage_->GetCFHandle(ColumnFamilyID::Metadata), iter.Key(), iter.Value()));
+    if (batch_sender.IsFull()) {
+      GET_OR_RET(sendMigrationBatch(&batch_sender));
+    }
 
     auto subkey_iter = iter.GetSubKeyIterator();
     if (!subkey_iter) {
@@ -1301,6 +1304,9 @@ Status SlotMigrator::sendSnapshotByRawKV() {
 
     for (subkey_iter->Seek(); subkey_iter->Valid(); subkey_iter->Next()) {
       GET_OR_RET(batch_sender.Put(subkey_iter->ColumnFamilyHandle(), subkey_iter->Key(), subkey_iter->Value()));
+      if (batch_sender.IsFull()) {
+        GET_OR_RET(sendMigrationBatch(&batch_sender));
+      }
 
       if (redis_type == RedisType::kRedisZSet) {
         InternalKey internal_key(subkey_iter->Key(), storage_->IsSlotIdEncoded());
@@ -1315,10 +1321,6 @@ Status SlotMigrator::sendSnapshotByRawKV() {
         GET_OR_RET(sendMigrationBatch(&batch_sender));
       }
     }
-
-    if (batch_sender.IsFull()) {
-      GET_OR_RET(sendMigrationBatch(&batch_sender));
-    }
   }
 
   GET_OR_RET(sendMigrationBatch(&batch_sender));
