@@ -540,9 +540,12 @@ class CommandFTTagVals : public Commander {
   Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     const auto &index_name = args_[1];
     const auto &tag_field_name = args_[2];
-    auto field_values = GET_OR_RET(srv->index_mgr.TagValues(ctx, index_name, tag_field_name, conn->GetNamespace()));
+    auto field_values = srv->index_mgr.TagValues(ctx, index_name, tag_field_name, conn->GetNamespace());
+    if (!field_values) {
+      return field_values;
+    }
 
-    std::vector<std::string> result_vec(field_values.begin(), field_values.end());
+    std::vector<std::string> result_vec(field_values->begin(), field_values->end());
 
     *output = conn->SetOfBulkStrings(result_vec);
 
