@@ -21,6 +21,7 @@
 #include "commander.h"
 
 #include "cluster/cluster_defs.h"
+#include "server/redis_reply.h"
 
 namespace redis {
 
@@ -47,9 +48,7 @@ std::string CommandTable::GetCommandInfo(const CommandAttributes *command_attrib
   command.append(redis::MultiLen(6));
   command.append(redis::BulkString(command_attributes->name));
   command.append(redis::Integer(command_attributes->arity));
-  command_flags.append(redis::MultiLen(1));
-  command_flags.append(redis::BulkString(command_attributes->InitialFlags() & kCmdWrite ? "write" : "readonly"));
-  command.append(command_flags);
+  command.append(redis::ArrayOfBulkStrings(CommandAttributes::FlagsToString(command_attributes->InitialFlags())));
   auto key_range = command_attributes->InitialKeyRange().ValueOr({0, 0, 0});
   command.append(redis::Integer(key_range.first_key));
   command.append(redis::Integer(key_range.last_key));
@@ -100,10 +99,7 @@ StatusOr<std::vector<int>> CommandTable::GetKeysFromCommand(const CommandAttribu
       [&](const std::vector<std::string> &, CommandKeyRange key_range) {
         key_range.ForEachKeyIndex([&](int i) { key_indexes.push_back(i); }, cmd_tokens.size());
       },
-      cmd_tokens,
-      [&](const auto &) {
-        status = {Status::NotOK, "The command has no key arguments"};
-      });
+      cmd_tokens, [&](const auto &) { status = {Status::NotOK, "The command has no key arguments"}; });
 
   if (!status) {
     return status;
