@@ -20,14 +20,13 @@
 
 #include "parse_util.h"
 
-#include <limits>
-
 #include "bit_util.h"
+#include "string_util.h"
 
-StatusOr<std::uint64_t> ParseSizeAndUnit(const std::string &v) {
-  auto [num, rest] = GET_OR_RET(TryParseInt<std::uint64_t>(v.c_str(), 10));
+StatusOr<std::uint64_t> ParseSizeAndUnit(std::string_view v) {
+  auto [num, rest] = GET_OR_RET(TryParseInt<std::uint64_t>(v, 10));
 
-  if (*rest == 0) {
+  if (rest == v.data() + v.size()) {
     return num;
   } else if (util::EqualICase(rest, "k")) {
     return util::CheckedShiftLeft(num, 10);
