@@ -20,6 +20,8 @@
 
 #pragma once
 
+#include <string_view>
+
 #include "ir.h"
 #include "search/ir_plan.h"
 
@@ -28,6 +30,7 @@ namespace kqir {
 struct Pass {
   virtual std::unique_ptr<Node> Transform(std::unique_ptr<Node> node) = 0;
 
+  virtual std::string_view Name() = 0;
   virtual void Reset() {}
 
   virtual ~Pass() = default;
@@ -87,7 +90,7 @@ struct Visitor : Pass {
       return Visit(std::move(v));
     } else if (auto v = Node::As<Sort>(std::move(node))) {
       return Visit(std::move(v));
-    } else if (auto v = Node::As<TopNSort>(std::move(node))) {
+    } else if (auto v = Node::As<TopN>(std::move(node))) {
       return Visit(std::move(v));
     } else if (auto v = Node::As<Projection>(std::move(node))) {
       return Visit(std::move(v));
@@ -95,7 +98,7 @@ struct Visitor : Pass {
       return Visit(std::move(v));
     }
 
-    __builtin_unreachable();
+    unreachable();
   }
 
   template <typename T>
@@ -220,7 +223,7 @@ struct Visitor : Pass {
     return node;
   }
 
-  virtual std::unique_ptr<Node> Visit(std::unique_ptr<TopNSort> node) {
+  virtual std::unique_ptr<Node> Visit(std::unique_ptr<TopN> node) {
     node->op = TransformAs<PlanOperator>(std::move(node->op));
     node->limit = VisitAs<LimitClause>(std::move(node->limit));
     node->order = VisitAs<SortByClause>(std::move(node->order));
