@@ -40,7 +40,7 @@ class IndexMetadata {
   uint8_t flag = 0;  // all reserved
   IndexOnDataType on_data_type;
 
-  const std::string &OnDataTypeName() const { return RedisTypeNames[(size_t)on_data_type]; }
+  std::string_view OnDataTypeName() const { return RedisTypeNames[(size_t)on_data_type]; }
 
   void Encode(std::string *dst) const {
     PutFixed8(dst, flag);
