@@ -53,8 +53,14 @@ enum RedisType : uint8_t {
   kRedisJson = 10,
   kRedisHyperLogLog = 11,
   kRedisTDigest = 12,
+  kRedisTimeSeries = 13,
+  kRedisTypeMax
 };
 
+inline constexpr const std::array<std::string_view, kRedisTypeMax> RedisTypeNames = {
+    "none",      "string", "hash",      "list",      "set",         "zset",      "bitmap",
+    "sortedint", "stream", "MBbloom--", "ReJSON-RL", "hyperloglog", "TDIS-TYPE", "timeseries"};
+
 struct RedisTypes {
   RedisTypes(std::initializer_list<RedisType> list) {
     for (auto type : list) {
@@ -94,10 +100,6 @@ enum RedisCommand {
   kRedisCmdLMove,
 };
 
-const std::vector<std::string> RedisTypeNames = {"none",      "string",      "hash",      "list",   "set",
-                                                 "zset",      "bitmap",      "sortedint", "stream", "MBbloom--",
-                                                 "ReJSON-RL", "hyperloglog", "TDIS-TYPE"};
-
 constexpr const char *kErrMsgWrongType = "WRONGTYPE Operation against a key holding the wrong kind of value";
 constexpr const char *kErrMsgKeyExpired = "the key was expired";
 
@@ -176,7 +178,7 @@ class Metadata {
   void PutExpire(std::string *dst) const;
 
   RedisType Type() const;
-  const std::string &TypeName() const;
+  std::string_view TypeName() const;
   size_t CommonEncodedSize() const;
   int64_t TTL() const;
   timeval Time() const;
@@ -364,3 +366,46 @@ class TDigestMetadata : public Metadata {
 
   double Delta() const { return 1. / static_cast<double>(compression); }
 };
+
+class TimeSeriesMetadata : public Metadata {
+ public:
+  enum class ChunkType : uint8_t {
+    UNCOMPRESSED = 0,
+    COMPRESSED = 1,
+  };
+
+  enum class DuplicatePolicy : uint8_t {
+    BLOCK = 0,
+    FIRST = 1,
+    LAST = 2,
+    MIN = 3,
+    MAX = 4,
+    SUM = 5,
+  };
+
+  uint64_t retention_time;
+  uint64_t chunk_size;
+  ChunkType chunk_type;
+  DuplicatePolicy duplicate_policy;
+  std::string source_key;
+  uint64_t last_timestamp = 0;  // Approximate last timestamp, used for compaction filter
+
+  explicit TimeSeriesMetadata(bool generate_version = true)
+      : Metadata(kRedisTimeSeries, generate_version),
+        retention_time(0),
+        chunk_size(0),
+        chunk_type(ChunkType::UNCOMPRESSED),
+        duplicate_policy(DuplicatePolicy::BLOCK) {}
+  TimeSeriesMetadata(uint64_t retention_time, uint64_t chunk_size, ChunkType chunk_type,
+                     DuplicatePolicy duplicate_policy, bool generate_version = true)
+      : Metadata(kRedisTimeSeries, generate_version),
+        retention_time(retention_time),
+        chunk_size(chunk_size),
+        chunk_type(chunk_type),
+        duplicate_policy(duplicate_policy) {}
+
+  void SetSourceKey(Slice key);
+
+  void Encode(std::string *dst) const override;
+  rocksdb::Status Decode(Slice *input) override;
+};
