@@ -282,9 +282,9 @@ struct JsonValue {
           value, path, [&insert_values, &results, index](const std::string & /*path*/, jsoncons::json &val) {
             if (val.is_array()) {
               auto len = static_cast<int64_t>(val.size());
-              // When index > 0, we need index < len
+              // When index >= 0, we need index <= len (allow insertion at end)
               // when index < 0, we need index >= -len.
-              if (index >= len || index < -len) {
+              if (index > len || index < -len) {
                 results.emplace_back(std::nullopt);
                 return;
               }
