@@ -26,7 +26,5 @@ Server *GetServer() { return nullptr; }
 
 int main(int argc, char **argv) {
   testing::InitGoogleTest(&argc, argv);
-  google::InitGoogleLogging(argv[0]);
-  google::InstallFailureSignalHandler();
   return RUN_ALL_TESTS();
 }
